@@ -14,9 +14,11 @@
 
 default_repos <- function() {
   opt <- getOption("repos")
+  was_list <- is.list(opt)
   if (! "CRAN" %in% names(opt) || opt[["CRAN"]] == "@CRAN@") {
     opt[["CRAN"]] <- "https://cloud.r-project.org"
   }
+  if (!was_list) opt <- unlist(opt)
   opt
 }
 
@@ -106,7 +108,7 @@ bold <- function(x) {
 }
 
 update_history <- function(cmd) {
-  tmp <- tempfile()
+  tmp <- tempfile("callr-hst-")
   on.exit(unlink(tmp, recursive = TRUE))
   utils::savehistory(tmp)
   cat(cmd, "\n", sep = "", file = tmp, append = TRUE)
